
library(stringr)


tetbranchhalfspacing <- 0.1


make.node <- function(label, brlen, lft=-1, rgt=-1, tt=-1, hybhgt=-1) {
  list(label=label, brlen=brlen, lft=lft, rgt=rgt, tt=tt, hybhgt=hybhgt, gfx=-1, hgt=-1)
  }


# test case
make.mnodes <- function() {
  nodes <- NULL
  nodes <- c(nodes, list(make.node("b",  .6)))     #1
  nodes <- c(nodes, list(make.node("y1", .4, tt=1, hybhgt=.1)))  #2
  nodes <- c(nodes, list(make.node("z1", .05)))                  #3
  nodes <- c(nodes, list(make.node("x1", .05)))                  #4
  nodes <- c(nodes, list(make.node("*",  .35, lft=3, rgt=4, tt=2, hybhgt=.2)))    #5
  nodes <- c(nodes, list(make.node("*",  .2, lft=2, rgt=5)))     #6
  nodes <- c(nodes, list(make.node("*",  .1, lft=1, rgt=6)))     #7
  nodes <- c(nodes, list(make.node("y0", .3, tt=1, hybhgt=.1)))  #8
  nodes <- c(nodes, list(make.node("z0", .05)))                   #9
  nodes <- c(nodes, list(make.node("x0", .05)))                  #10
  nodes <- c(nodes, list(make.node("*",  .25, lft=9, rgt=10, tt=2, hybhgt=.2)))    #11
  nodes <- c(nodes, list(make.node("*",  .2, lft=8, rgt=11)))     #12
  nodes <- c(nodes, list(make.node("a",  .5)))                   #13
  nodes <- c(nodes, list(make.node("*",  .2, lft=12, rgt=13)))     #14
  nodes <- c(nodes, list(make.node("*",  -1, lft=7, rgt=14)))    #15
  nodes
}

#make.mnodes <- function() {
#  nodes <- NULL
#  nodes <- c(nodes, list(make.node("b",  .6)))
#  nodes <- c(nodes, list(make.node("y1", .4, tt=1, hybhgt=.1)))
#  nodes <- c(nodes, list(make.node("z1", .4, tt=2, hybhgt=.2)))
#  nodes <- c(nodes, list(make.node("*",  .2, lft=2, rgt=3)))
#  nodes <- c(nodes, list(make.node("*",  .1, lft=1, rgt=4)))
#  nodes <- c(nodes, list(make.node("y0", .3, tt=1, hybhgt=.1)))
#  nodes <- c(nodes, list(make.node("z0", .3, tt=2, hybhgt=.2)))
#  nodes <- c(nodes, list(make.node("*",  .2, lft=6, rgt=7)))
#  nodes <- c(nodes, list(make.node("a",  .5)))
#  nodes <- c(nodes, list(make.node("*",  .2, lft=8, rgt=9)))
#  nodes <- c(nodes, list(make.node("*",  -1, lft=5, rgt=10)))
#  nodes
#}
#
extract.diphist.tetrees <- function(mnodes) {
  n <- length(mnodes)
  dnodes <- NULL
  tetrees <- NULL
  extract.subtrees(tetrees, dnodes, mnodes, n)
  }


extract.tetraploid.subtree <- function(tnodes, mnodes, n) {
  if (mnodes[[n]]$lft < 0) {                                                                            
    # add tip node as is, except label has 0 or 1 removed
    label <- str_sub(mnodes[[n]]$label, 1, str_length(mnodes[[n]]$label)-1)
    tnodes <- c(tnodes, list(make.node(label,  mnodes[[n]]$brlen)))  
  } else {
    # recurse
    tnodes <- extract.tetraploid.subtree(tnodes, mnodes, mnodes[[n]]$lft)
    lft <- length(tnodes)
    tnodes <- extract.tetraploid.subtree(tnodes, mnodes, mnodes[[n]]$rgt)
    rgt <- length(tnodes)  
    tnodes <- c(tnodes, list(make.node("*",  mnodes[[n]]$brlen, lft=lft, rgt=rgt)))  
  }
  tnodes
}


# converts MUL-tree to network = diphist plus tetratrees
extract.subtrees <- function(tetrees, dnodes, mnodes, n) {
  tt = mnodes[[n]]$tt
  
  if (tt > 0) {
    # add hyb-tip node at hybhgt
    dnodes <- c(dnodes, list(make.node("H",  mnodes[[n]]$brlen - mnodes[[n]]$hybhgt,
                tt=tt, hybhgt=mnodes[[n]]$hybhgt)))
    # check to see if already done the tetratree here...
    got <- FALSE
    for (i in 1:length(tetrees)) {
      if (identical(tetrees[[i]]$tt, tt)) {
        got <- TRUE
      }
    }
    # .. if not, recurse into tetratree and add to the list tetrees
    if (!got) {
      tnodes = NULL 
      tnodes <- extract.tetraploid.subtree(tnodes, mnodes, n)
      tetrees <- c(tetrees, list(list(tt=tt, nodes=tnodes)))
    }
  } else {
    if (mnodes[[n]]$lft < 0) {
      # add tip node as is
      dnodes <- c(dnodes, list(make.node(mnodes[[n]]$label,  mnodes[[n]]$brlen)))  
    } else { 
    # recurse within diphist
    lftsub <- extract.subtrees(tetrees, dnodes, mnodes, mnodes[[n]]$lft)
    dnodes <- lftsub$dnodes
    tetrees <- lftsub$tetrees
    lft <- length(dnodes)
    rgtsub <- extract.subtrees(tetrees, dnodes, mnodes, mnodes[[n]]$rgt)
    dnodes <- rgtsub$dnodes
    tetrees <- rgtsub$tetrees
    rgt <- length(dnodes)  
    dnodes <- c(dnodes, list(make.node("*",  mnodes[[n]]$brlen, lft=lft, rgt=rgt)))  
    }
  } 
  list(dnodes=dnodes, tetrees=tetrees)
}


find.node.suborder <- function(nodes, n, no) {
  if (nodes[[n]]$lft > 0) {
    no <- find.node.suborder(nodes, nodes[[n]]$lft, no)
    }
  no <- c(no, n)
  if (nodes[[n]]$rgt > 0) {
    no <- find.node.suborder(nodes, nodes[[n]]$rgt, no)
    }
  no
}


# make inorder (left-right) ordering of nodes
find.node.order <- function(nodes) {
  nodeorder <- integer(0)
  nodeorder <- find.node.suborder(nodes, length(nodes), nodeorder)
  nodeorder
}


# recursively convert branch lengths to node heights 
fillin.subtree.hgts <- function(nodes, n) {
  if (nodes[[n]]$lft > 0) {
    nodes <- fillin.subtree.hgts(nodes, nodes[[n]]$lft)
    nodes <- fillin.subtree.hgts(nodes, nodes[[n]]$rgt)
    nodes[[n]]$hgt <- nodes[[ nodes[[n]]$lft ]]$brlen  +  nodes[[ nodes[[n]]$lft ]]$hgt
  } else {
    if (nodes[[n]]$tt > 0) {
      nodes[[n]]$hgt <- nodes[[n]]$hybhgt
    } else {
      nodes[[n]]$hgt <- 0
    }
  }
  nodes
}


# recursively fill the graphics position of nodes (left-right if tree 
# root at bottom, top-bottom if leaves are on the right)
fillin.subtree.gfxs <- function(nodes, n) {
  if (nodes[[n]]$lft > 0) {
    nodes <- fillin.subtree.gfxs(nodes, nodes[[n]]$lft)
    nodes <- fillin.subtree.gfxs(nodes, nodes[[n]]$rgt)
    nodes[[n]]$gfx = 0.5 * (nodes[[ nodes[[n]]$lft ]]$gfx  +  nodes[[ nodes[[n]]$rgt ]]$gfx)
  }
  nodes
}

# fill the graphics position of tip nodes in diphist, then rest
fillin.dnode.gfxs <- function(dnodes, dnodeorder, tetrees) {
  gfx <- 1
  ttvisited <- rep(FALSE, length(tetrees))
  for (i in 1:length(dnodeorder)) {
    n <- dnodeorder[i]
    if (dnodes[[n]]$lft < 0) {
      dnodes[[n]]$gfx <- gfx
      tt <- dnodes[[n]]$tt
      if (tt > 0) {
        if (!ttvisited[tt]) {
          dnodes[[n]]$gfx <- dnodes[[n]]$gfx - .5
          gfx <- gfx + (length(tetrees[[tt]]$nodes) + 1) / 2
          ttvisited[tt] <- TRUE
        } else {
          gfx <- gfx + 1
        } 
      } else {
        gfx <- gfx + 1
      }
    }
  }
  dnodes <- fillin.subtree.gfxs(dnodes, length(dnodes))
  dnodes
}

# fill the graphics position of tip nodes in a tetratree, then rest
fillin.tnode.gfxs <- function(network, dnodeorder, tt, tnodeorder) {
  dnodes <- network$dnodes
  dnodegfx <- -1
  for (i in 1:length(dnodeorder)) {
    n <- dnodeorder[i]  
    if (dnodegfx < 0  && dnodes[[n]]$tt == tt) {
       dnodegfx <- dnodes[[n]]$gfx
    }
  }
  stopifnot(dnodegfx >0)
  gfx <- dnodegfx + .5
  tnodes <- network$tetrees[[tt]]$nodes
  for (i in 1:length(tnodeorder)) {
    n <- tnodeorder[i]
    if (tnodes[[n]]$lft < 0) {
      tnodes[[n]]$gfx <- gfx
      gfx <- gfx + 1
    }
  }  
  tnodes <- fillin.subtree.gfxs(tnodes, length(tnodes))
  tnodes
} 
    
  
  
  
print.node.index <- function(idx) {
  if (idx < 0) {
    cat("    ") 
  } else {
    cat(str_pad(idx, 4))
  } 
}
  
print.node <- function(nn, node) {
cat(str_pad(nn,4), str_pad(node$label,10), str_pad(node$brlen, 10))
print.node.index(node$lft)
print.node.index(node$rgt)
print.node.index(node$tt)
if (node$tt <0) {
    cat("          ") 
  } else {
    cat(str_pad(node$hybhgt, 10))
  } 
cat(str_pad(node$gfx, 10))
cat(str_pad(node$hgt, 10))
cat("\n")
}
  
  

print.diphist <- function(dnodes) {
  cat("Diploid history\n")
  cat(" idx      label      brlen lft rgt  tt    hybhgt       gfx       hgt\n")  
  for (nn in 1:length(dnodes)) {
    print.node(nn, dnodes[[nn]])
  }
}


print.tetree <- function(tt, tnodes) {
  cat("Tetraploid tree", tt, "\n")
  cat(" idx      label      brlen lft rgt                     gfx       hgt\n")  
  for (nn in 1:length(tnodes)) {
    print.node(nn, tnodes[[nn]])
  }
}


print.tetrees <- function(tetrees) {
  for (n in 1:length(tetrees)) {
    print.tetree(tetrees[[n]]$tt, tetrees[[n]]$nodes)
  }
}
  
  
print.diphist.tetrees <- function(network) {
print.diphist(network$dnodes)
print.tetrees(network$tetrees)
}
  

fillin.network.gfxs <- function(network) {
  dnodeorder <- find.node.order(network$dnodes)  
  network$dnodes <- fillin.dnode.gfxs(network$dnodes, dnodeorder, network$tetrees)
  for (i in 1:length(network$tetrees)) {
    tnodeorder <- find.node.order(network$tetrees[[i]]$nodes) 
    network$tetrees[[i]]$nodes <- fillin.tnode.gfxs(network, dnodeorder, i, tnodeorder) 
  }
  maxgfx <- network$dnodes[[ dnodeorder[length(dnodeorder)] ]]$gfx
  c(network, maxgfx=maxgfx)
}


fillin.network.hgts <- function(network) {
  network$dnodes <- fillin.subtree.hgts(network$dnodes, length(network$dnodes))
  for (i in 1:length(network$tetrees)) {
    tnodes <- network$tetrees[[i]]$nodes
    network$tetrees[[i]]$nodes <- fillin.subtree.hgts(tnodes, length(tnodes)) 
  }
  maxhgt <- network$dnodes[[length(network$dnodes)]]$hgt
  maxhgt
  c(network, maxhgt=maxhgt)
}



draw.branch.pair <- function(xr, yr, x1t, x2t, y1t, y2t, g) {
  lines(c(xr,xr,x1t), c(yr,y1t,y1t)-g)
  lines(c(xr,xr,x1t), c(yr,y1t,y1t)+g)
  lines(c(xr,xr,x2t), c(yr,y2t,y2t)-g)
  lines(c(xr,xr,x2t), c(yr,y2t,y2t)+g)
  }
  
  
draw.node.branch.pair <- function(node, g) {
}
  
  
draw.dnode.branch.pair <- function(dnodes, nn) {
  node <- dnodes[[nn]]
  nodelft <- dnodes[[node$lft]]
  nodergt <- dnodes[[node$rgt]]
  nodelft <- network$dnodes[[node$lft]]
  nodergt <- network$dnodes[[node$rgt]]
  draw.branch.pair(node$hgt, node$gfx, nodelft$hgt, nodergt$hgt, nodelft$gfx, nodergt$gfx, 0)
} 


draw.tnode.branch.pair <- function(tnodes, nn) {
  node <- tnodes[[nn]]
  nodelft <- tnodes[[node$lft]]
  nodergt <- tnodes[[node$rgt]]
  draw.branch.pair(node$hgt, node$gfx, nodelft$hgt, nodergt$hgt, nodelft$gfx, nodergt$gfx, tetbranchhalfspacing)
}


draw.root <- function(xr, yr, xt, g) {
  lines(c(xr,xt), c(yr,yr)-g)
  lines(c(xr,xt), c(yr,yr)+g)
  }

draw.hybline <- function(xh, yh1, yh2) {
  lines(c(xh,xh), c(yh1,yh2), lty=3)
}



draw.tet.root <- function(network, tt) {
  dnodes <- network$dnodes
  hybhgt <- -1
  hybtip1 <- -1
  hybtip2 <- -1
  for (n in 1:length(dnodes)) {
    if (dnodes[[n]]$tt == tt) {
       hybhgt <- dnodes[[n]]$hybhgt
       if (hybtip1 < 0) { hybtip1 <- n }  else { hybtip2 <- n }
    }
  }
  stopifnot (hybtip1 >0)
  stopifnot (hybtip2 >0)
  tnodes <- network$tetrees[[tt]]$nodes
  node <- tnodes[[length(tnodes)]]
  draw.root(hybhgt, node$gfx, node$hgt, tetbranchhalfspacing)
  draw.hybline(hybhgt, dnodes[[hybtip1]]$gfx, dnodes[[hybtip2]]$gfx)
  }


draw.tetra.tree <- function(network, tt) {
  maxhgt <- network$maxhgt
  draw.tet.root(network, tt)
  tnodes <- network$tetrees[[tt]]$nodes
  for (tn in 1:length(tnodes)) {
    if (tnodes[[tn]]$lft > 0) { 
      draw.tnode.branch.pair(tnodes, tn)
    } else {
      text(-.01*maxhgt, tnodes[[tn]]$gfx, labels=tnodes[[tn]]$label, adj=c(0,.3))
    }
  }
}



draw.network <- function(network) {
  maxhgt <- network$maxhgt
  plot(NULL, xlim=c(maxhgt,-.1*maxhgt), ylim=c(network$maxgfx,1), type="n")#, axes=FALSE, ann=FALSE)
  dnodes <- network$dnodes
  for (dn in 1:length(dnodes)) {
    if (dnodes[[dn]]$lft > 0) { 
      draw.dnode.branch.pair(dnodes, dn)
    } else {
      if (dnodes[[dn]]$tt < 0) {
        text(-.01*maxhgt, dnodes[[dn]]$gfx, labels=dnodes[[dn]]$label, adj=c(0,.3))
      }
    }
  }
  for (tt in 1:length(network$tetrees)) {
    draw.tetra.tree(network, tt)
  }
}  
  
  
mnodes <- make.mnodes() 
network <- extract.diphist.tetrees(mnodes)  
network <- fillin.network.gfxs(network)  
network <- fillin.network.hgts(network)  
print.diphist.tetrees(network)
draw.network(network)  















